###
#
# GETFUNCTIONSHASH() is used for checking whether this file has been played properly in DatabaseUpdate.php
#
DROP FUNCTION IF EXISTS GETFUNCTIONSHASH;
CREATE FUNCTION GETFUNCTIONSHASH()
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  RETURN '%%FUNCTIONSHASH%%';
END;


###
#
# QMORE(input, maxlen)
# inserts a span into `input` after `maxlen` number of characters and returns it.
#
DROP FUNCTION IF EXISTS QMORE;
CREATE FUNCTION QMORE(input TEXT, maxlen INT)
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  DECLARE output TEXT;
  IF maxlen < 1 THEN
    SET maxlen = 1;
  END IF;
  IF CHAR_LENGTH(input) > maxlen THEN
    SET output = CONCAT(INSERT(input, maxlen, 0, '<span class="qfq-more-text">'), '</span>');
  ELSE
    SET output = input;
  END IF;
  RETURN output;
END;


###
#
# QBAR(input)
# replaces '|' in `input` with '\|'
#
DROP FUNCTION IF EXISTS QBAR;
CREATE FUNCTION QBAR(input TEXT)
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  DECLARE output TEXT;
  SET output = REPLACE(input, '|', '\\|');
  RETURN output;
END;

###
#
# QNL2BR(input)
# replaces '|' in `input` with '\|'
#
DROP FUNCTION IF EXISTS QNL2BR;
CREATE FUNCTION QNL2BR(input TEXT)
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  DECLARE output TEXT;
  SET output = REPLACE(REPLACE(input, CHAR(13), ''), CHAR(10), '<br>');
  RETURN output;
END;

###
#
# QIFEMPTY(input, token)
# If 'input' is empty|0|0000-00-00|0000-00-00 00:00:00, replace by 'token'
#
DROP FUNCTION IF EXISTS QIFEMPTY;
CREATE FUNCTION QIFEMPTY(input TEXT, token TEXT)
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  DECLARE output TEXT;
  SET output =
      IF(ISNULL(input) OR input = '' OR input = '0' OR input = '0000-00-00' OR input = '0000-00-00 00:00:00', token,
         input);
  RETURN output;
END;

###
#
# strip_tags(input) - copied from https://stackoverflow.com/questions/2627940/remove-html-tags-from-record
#
DROP FUNCTION IF EXISTS strip_tags;
CREATE FUNCTION `strip_tags`(str TEXT)
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  DECLARE start, end INT DEFAULT 1;
  LOOP
    SET start = LOCATE("<", str, start);
    IF (!start) THEN RETURN str; END IF;
    SET end = LOCATE(">", str, start);
    IF (!end) THEN SET end = start; END IF;
    SET str = INSERT(str, start, end - start + 1, "");
  END LOOP;
END;

###
#
# QDATE_FORMAT(timestamp)
#
DROP FUNCTION IF EXISTS QDATE_FORMAT;
CREATE FUNCTION `QDATE_FORMAT`(ts DATETIME)
  RETURNS TEXT
  DETERMINISTIC
  SQL SECURITY INVOKER
BEGIN
  DECLARE output TEXT;
  SET output = IF(ts = 0, '-', DATE_FORMAT(ts, "%d.%m.%Y %H:%i"));
  RETURN output;
END;

